/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Oculta el contenido de las tablas en todos los diagramas y las
achica para que ocupen menos lugar. Accede directamente el
repositorio UML
- ENGLISH -
Hides tables content and reduces them, in diagrams. Requires
direct access to UML repository db.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING com.nomadesoft.ABLaUML.*.
USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.lib.*.
USING Progress.Lang.AppError.

CLASS com.nomadesoft.ABLaUML.DiagramCleaner:
  DEFINE PRIVATE VARIABLE cobUtils AS IUtils    NO-UNDO.
  DEFINE PRIVATE BUFFER cbfPackage FOR Repository.T_Package.

  /* nombres de los paquetes, se buscan los nombres nslib,clasif,project,dm,comps y menu y se utiliza el elemento que le sigue para nombrar los paquetes
  en el repositorio */
  DEFINE PUBLIC PROPERTY cchPackageNamesMap AS CHARACTER INITIAL 'lib,Lib,common,Common,project,Project,dm,Data View,menu,Menu View,comps,Components View' NO-UNDO
    GET.
    SET.

  METHOD PROTECTED CHARACTER getPackName(INPUT ipchName AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    minIndex = LOOKUP(ipchName,cchPackageNamesMap).
    IF minIndex GT 0 THEN
      RETURN ENTRY(minIndex + 1,cchPackageNamesMap).
    RETURN ipchName.
  END METHOD.

  DEFINE PROTECTED PROPERTY cinProjectPackageId AS INTEGER NO-UNDO
    GET:
      IF cinProjectPackageId EQ 0 THEN DO:
        cinProjectPackageId = cobUtils:findPackage(getPackName('project'),0).
        IF cinProjectPackageId EQ 0 THEN
          cinProjectPackageId = cobUtils:createModelPackage(getPackName('project'),0,?).
      END.
      RETURN cinProjectPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinProjectComponentsPackageId AS INTEGER NO-UNDO
    GET:
      IF cinProjectComponentsPackageId EQ 0 THEN DO:
        cinProjectComponentsPackageId = cobUtils:findPackage(getPackName('comps'),cinProjectPackageId).
        IF cinProjectComponentsPackageId EQ 0 THEN
          cinProjectComponentsPackageId = cobUtils:createModelPackage(getPackName('comps'),cinProjectPackageId,4).
      END.
      RETURN cinProjectComponentsPackageId.
    END GET.
    PRIVATE SET.

  DEFINE PROTECTED PROPERTY cinDataModelPackageId AS INTEGER NO-UNDO
    GET:
      IF cinDataModelPackageId EQ 0 THEN DO:
        cinDataModelPackageId = cobUtils:findPackage(getPackName('dm'),cinProjectPackageId).
        IF cinDataModelPackageId EQ 0 THEN
          cinDataModelPackageId = cobUtils:createModelPackage(getPackName('dm'),cinProjectPackageId,3).
      END.
      RETURN cinDataModelPackageId.
    END GET.
    PRIVATE SET.

  CONSTRUCTOR PUBLIC DiagramCleaner(INPUT ipobLogger AS CLASS ILogger):
    cobUtils = NEW EAUtils(TRUE).
  END CONSTRUCTOR.

  /** destructor **/
  DESTRUCTOR DiagramCleaner():
    DELETE OBJECT cobUtils.
  END DESTRUCTOR.

  METHOD PRIVATE LOGICAL EsModeloDeDatos(INPUT ipinPackageID AS INTEGER):
    IF ipinPackageID EQ cinDataModelPackageId THEN
      RETURN TRUE.
    FIND cbfPackage NO-LOCK WHERE cbfPackage.Package_Id EQ ipinPackageID.
    IF cbfPackage.PARENT_ID EQ cinDataModelPackageId THEN
      RETURN TRUE.
    IF cbfPackage.PARENT_ID EQ 0 OR cbfPackage.PARENT_ID EQ ?  THEN
      RETURN FALSE.
    RETURN EsModeloDeDatos(cbfPackage.PARENT_ID).
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
      RETURN FALSE.
    END CATCH.
  END METHOD.

  METHOD PRIVATE LOGICAL EsProyecto(INPUT ipinPackageID AS INTEGER):
    IF ipinPackageID EQ cinProjectComponentsPackageId THEN
      RETURN TRUE.
    FIND cbfPackage NO-LOCK WHERE cbfPackage.Package_Id EQ ipinPackageID.
    IF cbfPackage.PARENT_ID EQ cinProjectComponentsPackageId THEN
      RETURN TRUE.
    IF cbfPackage.PARENT_ID EQ 0 OR cbfPackage.PARENT_ID EQ ?  THEN
      RETURN FALSE.
    RETURN EsProyecto(cbfPackage.PARENT_ID).
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
      RETURN FALSE.
    END CATCH.
  END METHOD.

  /* dada una lista de opciones, separadas por ';', establece la visibilidad de la opcin indicada */
  METHOD PROTECTED CHARACTER SetVisibleOption(INPUT ipchData AS CHARACTER,INPUT ipchOption AS CHARACTER,INPUT iplgVisible AS LOGICAL):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchVisible AS CHARACTER  INITIAL '=0' NO-UNDO.
    DEFINE VARIABLE mchNotVisible AS CHARACTER  INITIAL '=1' NO-UNDO.

    IF iplgVisible THEN ASSIGN
      mchNotVisible = '=0'
      mchVisible = '=1'
      .

    IF ipchData EQ ? OR ipchData EQ '' THEN
      RETURN ipchOption + mchVisible.
    minIndex = LOOKUP(ipchOption + mchNotVisible,ipchData,';').
    IF minIndex GT 0 THEN
      ENTRY(minIndex,ipchData,';') = ipchOption + mchVisible.
    ELSE DO:
      minIndex = LOOKUP(ipchOption + mchVisible,ipchData,';').
      IF minIndex EQ 0 THEN
        ipchData = ipchData + ';' + ipchOption + mchVisible.
    END.
    RETURN ipchData.
  END METHOD.

  /* oculta atributos y operaciones para todos los diagramas */
  METHOD PROTECTED VOID HideAttOps():
    DEFINE BUFFER mbfDiag FOR Repository.T_Diagram.
    FOR EACH mbfDiag:
      IF NOT EsProyecto(Package_id) THEN
        NEXT.
      mbfDiag.PData = SetVisibleOption(SetVisibleOption(mbfDiag.PData,'HideAtts',TRUE),'HideOps',TRUE).
    END.
  END METHOD.

  /* reduce el tamao de las tablas en todos los diagramas */
  METHOD PROTECTED VOID ShrinkTables():
    DEFINE BUFFER mbfDO FOR Repository.T_DiagramObjects.
    DEFINE BUFFER mbfObj FOR Repository.T_Object.
    DEFINE BUFFER mbfDiag FOR Repository.T_Diagram.
    DEFINE VARIABLE mlgProcessPack AS LOGICAL  NO-UNDO.
    DEFINE VARIABLE mlgProcessDiag AS LOGICAL  NO-UNDO.

    FOR EACH mbfObj BREAK BY mbfObj.Package_Id:
      IF FIRST-OF(Package_ID) THEN
        mlgProcessPack = EsModeloDeDatos(Package_ID).
      IF NOT mlgProcessPack THEN
        NEXT.
      FOR EACH mbfDO WHERE mbfDO.OBJECT_ID EQ mbfObj.OBJECT_ID BREAK BY mbfDO.Diagram_ID:
        IF FIRST-OF(Diagram_ID) THEN DO:
          FIND mbfDiag WHERE mbfDiag.Diagram_iD EQ mbfDO.Diagram_ID.
          mlgProcessDiag = EsProyecto(mbfDiag.Package_ID).
        END.
        IF NOT mlgProcessDiag THEN
          NEXT.
        ASSIGN
          mbfDO.RectBottom = mbfDO.RectTop - 40
          mbfDO.RectRight = mbfDO.RectLeft + 140
          .
      END.
    END.
  END METHOD.

  /* busca todos los objetos (tablas) del modelo de datos en los diagramas y les reduce el tamao.
    busca todos los diagramas y les oculta atributos y operaciones */
  METHOD PUBLIC VOID CleanIt():
    HideAttOps().
    ShrinkTables().
  END METHOD.
END CLASS.
