ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.ABLaUML.DDWorker.

/**
Se encarga de crear todos los objetos y paquetes necesarios para luego poder procesar las relaciones
entre ellos.
**/
CLASS com.nomadesoft.ABLaUML.FullDDWorker INHERITS DDWorker:
  DEFINE PROTECTED PROPERTY cinDBPackageID AS INTEGER NO-UNDO
    GET.
    SET.

  DEFINE PROTECTED PROPERTY cinTableID AS INTEGER NO-UNDO
    GET.
    SET.
  DEFINE PROTECTED PROPERTY cinIndexID AS INTEGER INITIAL ? NO-UNDO
    GET.
    SET.


  DEFINE PUBLIC PROPERTY clgCreateTableTags AS LOGICAL INITIAL TRUE NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY clgCreateFieldTags AS LOGICAL NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY clgCreateIndexTags AS LOGICAL NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY clgCreateSequenceTags AS LOGICAL INITIAL TRUE NO-UNDO
    GET.
    SET.

  CONSTRUCTOR FullDDWorker(INPUT ipobUtils AS CLASS IUtils):
    SUPER(ipobUtils).
  END CONSTRUCTOR.

  /* mtodos que los hijos sobrescribirn para hacer el trabajo real, invocados desde la implementacin de IXMLWorker */
  METHOD PROTECTED OVERRIDE VOID DatabaseStart():
    cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Comienza base de datos &1',cobParser:AttributeValue('name')),'DEBUG').
    cinDBPackageId = cobUtils:CreatePackage (
        input cobParser:AttributeValue('name'),
        input cinDataModelPackageID
        ). 
    if cinDBPackageID eq ? or cinDBPackageID eq 0 then
      undo, THROW NEW AppError("Failure to Create Package for " + cobParser:AttributeValue('name'),1).
                                                 /* Create corresponding object */
    cobUtils:CreateObject (
      input "Package",
      input cobParser:AttributeValue('name'),
      input cinDBPackageID,
      input cinDataModelPackageID,
      input "nsDatabase",
      input "",
      input ""
      ).
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID TableStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Tabla &1',cobParser:AttributeValue('name')),'DEBUG').
    minObjectID = cobUtils:CreateObject (
        input "Class",
        input cobParser:AttributeValue('name'),
        input cinDBPackageID,
        input 0,
        input "nsTable",                                     /* Should be oeTable */
        input cobParser:AttributeValue('desc'),
        input ""
        ).
      if     minObjectID eq ? or minObjectID eq 0
      then
        cobParser:DoLog(string(this-object),"Failure to Create Object for Table " + cobParser:AttributeValue('name'),'WARNING' ).
                                                          /* Add tags for table */
      cinTableID = minObjectId.
      IF clgCreateTableTags THEN
        CreateObjectTags(minObjectId,cchTableTagsMap).
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID FieldStart():
    DEFINE VARIABLE mchStereotype AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minAID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchInitial AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchIFname AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchNotas AS CHARACTER   NO-UNDO.
    
    ASSIGN
      mchIFname = cobParser:AttributeValue('name')
      mchInitial = cobParser:AttributeValue('initial')
      .
    IF mchInitial EQ ? THEN
      mchInitial = ''.
    IF cinIndexID NE ? THEN DO:
      IF mchIFname EQ ? OR mchIFName EQ '' THEN /* es un ndice por omisin, que no tiene campos en realidad */
        RETURN.
      cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Campo &1 para ndice',mchIFName),'DEBUG').
      if cobParser:AttributeValue('descending') EQ 'true' 
      THEN
        mchStereotype = 'Desc'.
      ELSE
        mchStereotype = 'Asc'.
      if cobParser:AttributeValue('abbreviate') EQ 'true'
      then
        mchStereotype = mchStereotype + "Abv".
      cobUtils:CreateOperationParam (
          input cinIndexID,
          input mchIFName,
          input UPPER(cobParser:AttributeValue('type')),
          input mchInitial,
          input INTEGER(cobParser:AttributeValue('seq')) - 1,
          input "in",
          input mchStereotype
          ).
    END.
    ELSE DO:
        cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Campo &1',mchIFname),'DEBUG').
        mchNotas = cobParser:AttributeValue('help').
        IF mchNotas EQ ? THEN
          mchNotas = cobParser:AttributeValue('desc').
        minAID = cobUtils:CreateAttribute (
            input cinTableID,
            input mchIFname,
            input "nsColumn",
            input mchNotas,
            input INTEGER(cobParser:AttributeValue('order')),
            input (IF cobParser:AttributeValue('mandatory') EQ 'true' THEN 1 ELSE 0),
            input mchInitial,
            input cobParser:AttributeValue('dtype'),
            input "AttribTable"
            ).
        IF cobParser:AttributeValue('label') NE ? THEN
          cobUtils:setAttributeAlias(
              minAID,
              cobParser:AttributeValue('label')
              ).
                                                          /* Add tags for columns */
       IF clgCreateFieldTags THEN
        CreateAttributeTags(minAID,cchColumnTagsMap).
    END.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID IndexStart():
    DEFINE VARIABLE mchIndexType AS CHARACTER   NO-UNDO.

    cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('ndice &1',cobParser:AttributeValue('name')),'DEBUG').
    if cobParser:AttributeValue('primary') EQ 'true'
    then do:
      mchIndexType = "nsPk".
      /* crear atributo en la tabla */
      cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Es ndice primario',cobParser:AttributeValue('name')),'DEBUG').
      IF clgCreateTableTags THEN
        CreateObjectTags(cinTableID,'name,PrimaryIndex').
    END.
    else mchIndexType = "nsIndex".

    cinIndexID = cobUtils:CreateOperation (
        input cinTableID,
        input cobParser:AttributeValue('name'),
        input mchIndexType
        ).
    if cobParser:AttributeValue('unique') EQ 'true'
    then do:
      cobUtils:CreateOperationPost (
          input cinIndexID,
          input "IsUnique", 
          input "Functional"
          ).
    end.
    IF clgCreateIndexTags THEN
      cobUtils:SetOperationTag (
          input cinIndexID,
          input "IsWordIndex", 
          input (IF cobParser:AttributeValue('wordindex') EQ 'true' THEN 'YES' ELSE 'NO')
          ).
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID StorageAreaStart():
    cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('rea de almacenamiento &1',cobParser:AttributeValue('name')),'DEBUG').
    IF clgCreateTableTags THEN
      CreateObjectTags(cinTableID,'name,AreaName').
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID SequenceStart():
    DEFINE VARIABLE minSeqPackID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE minSeqID AS INTEGER     NO-UNDO.

    cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Secuencia &1',cobParser:AttributeValue('name')),'DEBUG').
    minSeqPackID = cobUtils:FindPackage("Secuencias",cinDBPackageID).
    IF minSeqPackID EQ ? THEN DO:
      cobParser:DoLog(STRING(THIS-OBJECT),SUBSTITUTE('Creacin del paquete de secuencias',cobParser:AttributeValue('name')),'DEBUG').
      minSeqPackID = cobUtils:CreatePackage (
        input "Secuencias",
        input cinDBPackageID
        ).
      cobUtils:CreateObject (
        input "Package",
        input "Secuencias",
        input minSeqPackID,
        input cinDBPackageID,
        input "",
        input "",
        input ""
        ).
    END.

    minSeqID = cobUtils:CreateObject(
        input "Class",
        input cobParser:AttributeValue('name'),
        input minSeqPackID,
        input 0,
        input "nsSequence",
        input "",
        input ""
        ).
                                                      /* Add tags for sequences */
    IF clgCreateSequenceTags THEN
      CreateObjectTags(minSeqID,cchSequenceTagsMap).
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID DatabaseEnd():
    cinDBPackageId = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID TableEnd():
    cinTableID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID IndexEnd():
    cinIndexID = ?.
  END METHOD.
END CLASS.
