/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
Cdigo original tomado de NSRA.
---
autor original: I. Vasquez (ivasquez@nomadesoft.com.ar)
adaptacin: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase para generar informacin de registro.
- ENGLISH -
Log generation class.
*****************/

ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING com.nomadesoft.lib.ILogger.

/**
 * Logger.
 * Este objeto permite implementar un registro de Logger. Que es guardado en un fichero.
 */
CLASS com.nomadesoft.lib.Logger IMPLEMENTS ILogger:
  /**
   * Un Stream que es propia de cada objeto.
   */
  DEFINE PRIVATE STREAM cstLgStream.
  DEFINE PRIVATE VARIABLE clgStreamValid AS LOGICAL     NO-UNDO.

  /**
   * Todas las constantens que representan los niveles del Logger.
   */
  DEFINE PUBLIC PROPERTY ALL_LEVEL AS INTEGER INITIAL 0 NO-UNDO GET.
  DEFINE PUBLIC PROPERTY DEBUG_LEVEL AS INTEGER INITIAL 0 NO-UNDO GET.
  DEFINE PUBLIC PROPERTY ERROR_LEVEL AS INTEGER INITIAL 10 NO-UNDO GET.
  DEFINE PUBLIC PROPERTY WARNING_LEVEL AS INTEGER INITIAL 20 NO-UNDO GET.
  DEFINE PUBLIC PROPERTY NOTICE_LEVEL AS INTEGER INITIAL 30 NO-UNDO GET.
  DEFINE PUBLIC PROPERTY MESSAGE_LEVEL AS INTEGER INITIAL 40 NO-UNDO GET.
  DEFINE PUBLIC PROPERTY NONE_LEVEL AS INTEGER INITIAL 100 NO-UNDO GET.

  /**
   * Indica si con cada salida debe mostrar la hora o no
   */
  DEFINE PUBLIC PROPERTY clgShowTime AS LOGICAL INITIAL TRUE NO-UNDO
    GET.
    SET.

  /**
   * El nivel actual del Logger.
   */
  DEFINE PUBLIC PROPERTY cinLevel AS INTEGER INITIAL 0 NO-UNDO
    GET.
    SET(INPUT ipinLevel AS INTEGER):
        IF ipinLevel >= 0 AND ipinLevel <= 100 THEN
          cinLevel = ipinLevel.
    END SET.

  /**
   * El nombre del fichero log.
   */
  DEFINE PUBLIC PROPERTY cchFileName AS CHARACTER INITIAL 'debug.log' NO-UNDO
    GET.
    PROTECTED SET(INPUT ipchFileName AS CHARACTER):
        IF ipchFileName <> '' THEN
          cchFileName = ipchFileName.
        ELSE
          cchFileName = 'debug.log'.
    END SET.

  /**
   * El directorio base donde se guarda el fichero log.
   * Solo se usan direcciones de fichero UNIX.
   */
  DEFINE PROTECTED PROPERTY cchDirBaseName AS CHARACTER INITIAL '' NO-UNDO
    GET.
    SET(INPUT ipchDirBaseName AS CHARACTER):
        IF ipchDirBaseName <> ? THEN
          cchDirBaseName = ipchDirBaseName.
        ELSE
          cchDirBaseName = ''.
    END SET.

  /**
   * Constructor.
   * Abre el Stream.
   */
  CONSTRUCTOR PUBLIC Logger():
    OpenOut().
  END CONSTRUCTOR.

  /**
   * Constructor.
   * Abre el Stream.
   */
  CONSTRUCTOR PUBLIC Logger(INPUT ipinLevel AS INTEGER):
    cinLevel = ipinLevel.
    OpenOut().
  END CONSTRUCTOR.

  /**
   * Constructor.
   * Abre el Stream.
   */
  CONSTRUCTOR PUBLIC Logger(INPUT ipinLevel AS INTEGER, INPUT ipchFileName AS CHARACTER):  
    cinLevel = ipinLevel.
    cchFileName = ipchFileName.
    OpenOut().
  END CONSTRUCTOR.

  /**
   * Constructor.
   * Abre el Stream.
   */
  CONSTRUCTOR PUBLIC Logger(INPUT ipinLevel AS INTEGER, INPUT ipchDirBaseName AS CHARACTER, INPUT ipchFileName AS CHARACTER):
    cinLevel = ipinLevel.
    cchDirBaseName = ipchDirBaseName.
    cchFileName = ipchFileName.
    OpenOut().
  END CONSTRUCTOR.

  /**
   * Constructor.
   * Abre el Stream.
   */
  CONSTRUCTOR PUBLIC Logger(INPUT ipchFileName AS CHARACTER):
    cchFileName = ipchFileName.
    OpenOut().
  END CONSTRUCTOR.

  /**
   * Constructor.
   * Abre el Stream.
   */
  CONSTRUCTOR PUBLIC Logger(INPUT ipchDirBaseName AS CHARACTER, INPUT ipchFileName AS CHARACTER):
    cchDirBaseName = ipchDirBaseName.
    cchFileName = ipchFileName.
    OpenOut().
  END CONSTRUCTOR.

  /**
   * Destructor.
   * Cierra el Stream.
   */
  DESTRUCTOR PUBLIC Logger():
    CloseOut().
  END DESTRUCTOR.

  /**
   * Open el Stream.
   * Y coloca un mensaje en el fichero.
   */
  METHOD PROTECTED VOID OpenOut():
    clgStreamValid = FALSE.
    OUTPUT STREAM cstLgStream TO VALUE(GetFileOutName()) APPEND UNBUFFERED.
    PUT STREAM cstLgStream UNFORMATTED 'UP LOGGER: ' + GetFileOutName() + ' - ' + STRING(NOW) SKIP.
    clgStreamValid = TRUE.
    CATCH sError AS Progress.Lang.SysError:
      DELETE OBJECT sError.
    END CATCH.
  END METHOD.

  /**
   * Cierra el Stream.
   * Y coloca un mensaje antes en el fichero.
   */
  METHOD PROTECTED VOID CloseOut():
    IF NOT clgStreamValid THEN
      RETURN.
    PUT STREAM cstLgStream UNFORMATTED 'DOWN LOGGER: ' + GetFileOutName() + ' - ' + STRING(NOW) SKIP.
    OUTPUT STREAM cstLgStream CLOSE.
  END METHOD.

  /**
   * Escribe la etiqueta de comienzo de cada linea.
   */
  METHOD PROTECTED VOID OutPutTag(INPUT ipinLevel AS INTEGER):
    IF NOT clgStreamValid THEN RETURN.
    PUT STREAM cstLgStream UNFORMATTED GetLevelName(ipinLevel) + (IF clgShowTime THEN '(' + STRING(TIME,'HH:MM:SS') + '): ' ELSE ': ').
  END METHOD.

  /**
   * Retorna el nombre completo del fichero de log.
   */
  METHOD PUBLIC CHARACTER GetFileOutName():
    DEFINE VARIABLE mchDir AS CHARACTER   NO-UNDO.
    mchDir = cchDirBaseName.
    IF LENGTH(mchDir) GT 0 AND SUBSTRING(mchDir,1,1) <> '/' THEN DO:
      IF OPSYS EQ 'WIN32' THEN DO:
        mchDir = SEARCH('Logger.cls').
        mchDir = REPLACE(mchDir,'~\','/').
      END.
      ELSE
        mchDir = SEARCH('Logger.cls').
      mchDir = SUBSTRING(mchDir,1,R-INDEX(mchDir,'/') - 1). /*quito Logger.cls*/
      mchDir = SUBSTRING(mchDir,1,R-INDEX(mchDir,'/') - 1). /*quito utiles*/
      IF LENGTH(cchDirBaseName) > 0 THEN DO:
        mchDir = SUBSTRING(mchDir,1,R-INDEX(mchDir,'/')). /*quito nsra, sin quitar la barra*/
        mchDir = mchDir + cchDirBaseName.
      END.
      ELSE
        mchDir = mchDir + '/servicios/log'. /* agrego la ruta al directorio de log */
    END.
    RETURN mchDir + '/' + cchFileName.
  END METHOD.

/**
   * Retorna nombre del Nivel Actual.
   */
  METHOD PUBLIC CHARACTER GetLevelName():
    RETURN GetLevelName(cinLevel).
  END METHOD.

  /**
   * Retorna el nombre de un Nivel especifico.
   */
  METHOD PUBLIC CHARACTER GetLevelName(INPUT ipinLevel AS INTEGER):
    IF ipinlevel = ERROR_LEVEL THEN RETURN 'ERROR'.
    IF ipinlevel = WARNING_LEVEL THEN RETURN 'WARNING'.
    IF ipinlevel = NOTICE_LEVEL THEN RETURN 'NOTICE'.
    IF ipinlevel = MESSAGE_LEVEL THEN RETURN 'MESSAGE'.
    
    RETURN 'DEBUG'.
  END METHOD.


  /**
   * Retorna True si un numero de nivel especifico esta en el rango de los niveles de salida.
   */
  METHOD PUBLIC LOGICAL CanLevelOut(INPUT ipinLevel AS INTEGER):
    RETURN ipinLevel >= cinLevel AND ipinLevel <= NONE_LEVEL.
  END METHOD.
 
  METHOD PUBLIC VOID SetLevel(INPUT ipinLevel AS INTEGER):
    cinLevel = ipinLevel.
  END METHOD.

  METHOD PUBLIC INTEGER GetLevel(INPUT ipchLevel AS CHARACTER):
    CASE ipchLevel:
      WHEN 'WARNING' THEN RETURN WARNING_LEVEL.
      WHEN 'ERROR' THEN RETURN ERROR_LEVEL.
      WHEN 'NOTICE' THEN RETURN NOTICE_LEVEL.
      WHEN 'MESSAGE' THEN RETURN MESSAGE_LEVEL.
      OTHERWISE RETURN DEBUG_LEVEL.
    END CASE.
  END METHOD.
  /**
   * Imprime un mensaje en le fichero.
   */
  METHOD PUBLIC VOID OutLog(INPUT ipchMessage AS CHARACTER):
    OutLog(ipchMessage,cinLevel).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipchMessage AS CHARACTER, INPUT ipinLevel AS INTEGER):
    IF NOT clgStreamValid THEN RETURN.
    IF NOT CanLevelOut(ipinLevel) THEN RETURN.
    OutPutTag(ipinLevel).
    PUT STREAM cstLgStream UNFORMATTED ipchMessage SKIP.
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipdaMessage AS DATE):
    OutLog(STRING(ipdaMessage)).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipdaMessage AS DATE, INPUT ipinLevel AS INTEGER):
    OutLog(STRING(ipdaMessage),ipinLevel).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipdtMessage AS DATETIME):
    OutLog(STRING(ipdtMessage)).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipdtMessage AS DATETIME, INPUT ipinLevel AS INTEGER):
    OutLog(STRING(ipdtMessage),ipinLevel).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipdeMessage AS DECIMAL):
    OutLog(STRING(ipdeMessage)).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipdeMessage AS DECIMAL, INPUT ipinLevel AS INTEGER):
    OutLog(STRING(ipdeMessage),ipinLevel).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipinMessage AS INT64):
    OutLog(STRING(ipinMessage)).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipinMessage AS INT64, INPUT ipinLevel AS INTEGER):
    OutLog(STRING(ipinMessage),ipinLevel).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipinMessage AS INTEGER):
    OutLog(STRING(ipinMessage)).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT ipinMessage AS INTEGER, INPUT ipinLevel AS INTEGER):
    OutLog(STRING(ipinMessage),ipinLevel).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT iplgMessage AS LOGICAL):
    OutLog(STRING(iplgMessage)).
  END METHOD.

  METHOD PUBLIC VOID OutLog(INPUT iplgMessage AS LOGICAL, INPUT ipinLevel AS INTEGER):
    OutLog(STRING(iplgMessage),ipinLevel).
  END METHOD.
  /** End OutLog */
END CLASS.
