/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor original: T. M. Hursh (thomas@cintegrity.com)
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Genera el xml de la definicin (estructura) de la base de datos
asociada al Alias DictSource.
- ENGLISH -
Generates xml of database structure, for the database associated
to DictSource Alias.
*****************/

ROUTINE-LEVEL ON ERROR UNDO, THROW.
USING com.nomadesoft.lib.XMLwriter.
USING com.nomadesoft.lib.ILogger.


CLASS com.nomadesoft.xml.DDictwriter INHERITS XMLwriter:
  DEFINE PRIVATE VARIABLE criDB AS RECID NO-UNDO.

  DEFINE PUBLIC PROPERTY cchDBName AS CHARACTER INITIAL ? NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY cobLogger AS ILogger NO-UNDO
    GET.
    SET.

  METHOD PRIVATE VOID DoLog(INPUT ipchData AS CHARACTER,INPUT ipchLevel AS CHARACTER):
    IF VALID-OBJECT(cobLogger) THEN
      cobLogger:OutLog(ipchData,cobLogger:getLevel(ipchLevel)).
  END METHOD.

  METHOD PROTECTED VOID AddStorageArea(INPUT ipinFileNum AS INTEGER):
    define buffer mbfStorageObject for DictSource._StorageObject.
    define buffer mbfArea for DictSource._Area.

    DoLog('Determinando area de almacenamiento..','DEBUG').
    find mbfStorageObject no-lock
          where mbfStorageObject._DB-Recid eq criDB
          and   mbfStorageObject._Object-Type eq 1
          and   mbfStorageObject._Object-Number eq ipinFileNum
          no-error.
    if available( mbfStorageObject )
    then do:
      find mbfArea no-lock
          where mbfARea._Area-Number eq mbfStorageObject._Area-Number
          no-error.
      if available( mbfArea )
      THEN
        OutputTag('storagearea','name,' + mbfArea._Area-Name,?).
    end.
  END METHOD.

  METHOD PROTECTED VOID AddIndexFields(INPUT ipriIndex AS RECID):
    define buffer mbfIndexField for DictSource._Index-Field.
    define buffer mbfField for DictSource._Field.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.
  
    for each mbfIndexField no-lock
        where mbfIndexField._Index-Recid EQ ipriIndex
        ON ERROR UNDO, THROW:
      find mbfField no-lock where mbfIndexField._Field-Recid = recid(mbfField).
      mchAtts = 'name,' + Escapar(mbfField._Field-Name) + ',type,' + mbfField._Data-Type + ',seq,' + STRING(mbfIndexField._Index-Seq).
      IF mbfField._Initial NE ? THEN
        mchAtts = mchAtts + ',initial,' + Escapar(mbfField._Initial).
      if NOT mbfIndexField._Ascending THEN
        mchAtts = mchAtts + ',descending,true'.
      if mbfIndexField._Abbreviate then
        mchAtts = mchAtts + ',abbreviate,true'.
      OutputTag('field',mchAtts,?).
    end.
  END METHOD.

  METHOD PROTECTED VOID AddIndexes(INPUT ipriTable AS RECID,INPUT ipriPKIndex AS RECID):
    define buffer mbfIndex for DictSource._Index.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.

    for each mbfIndex no-lock
        where mbfIndex._File-Recid EQ ipriTable
        ON ERROR UNDO, THROW:
      DoLog('procesando ndice ' + mbfIndex._Index-Name,'DEBUG').
      mchAtts = 'name,' + Escapar(mbfIndex._Index-Name).
      if recid( mbfIndex ) EQ ipriPKIndex then
        mchAtts = mchAtts + ',primary,true'.
      if mbfIndex._Unique then    
        mchAtts = mchAtts + ',unique,true'.
      if mbfIndex._WordIdx NE ? THEN
        mchAtts = mchAtts + ',wordindex,true'.
      StartTag('index',mchAtts).
      DoLog('agregando campos..','DEBUG').
      AddIndexFields(RECID(mbfIndex)).
      EndTag().
      DoLog('ndice completo','DEBUG').
    end.
  END METHOD.

  METHOD PROTECTED VOID AddFields(INPUT ipriTable AS RECID):
    define buffer mbfField for DictSource._Field.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.
    for each mbfField no-lock
        where mbfField._File-Recid eq ipriTable
        by mbfField._Order
        ON ERROR UNDO, THROW:
      DoLog('campo ' + mbfField._Field-name,'DEBUG').
      mchAtts = 'name,' + Escapar(mbfField._Field-name) + ',dtype,' + mbfField._Data-type.
      IF mbfField._Order NE ? THEN
        mchAtts = mchAtts + ',order,' + STRING(mbfField._Order).
      IF mbfField._Desc NE ? THEN
        mchAtts = mchAtts + ',desc,' + Escapar(mbfField._Desc).
      IF mbfField._Initial NE ? THEN
        mchAtts = mchAtts + ',initial,' + Escapar(mbfField._Initial).
      if mbfField._Mandatory THEN
        mchAtts = mchAtts + ',mandatory,true'.
      IF mbfField._Fld-Case THEN
        mchAtts = mchAtts + ',sensitive,true'.
      IF mbfField._Sys-Field THEN
        mchAtts = mchAtts + ',sysfield,true'.
      IF mbfField._Col-Label NE ? THEN
        mchAtts = mchAtts + ',collabel,' + Escapar(mbfField._Col-Label).
      IF mbfField._Col-Label-SA NE ? THEN
        mchAtts = mchAtts + ',collabelsa,' + Escapar(mbfField._Col-Label-SA).
      IF mbfField._Help NE ? THEN
        mchAtts = mchAtts + ',help,' + Escapar(mbfField._Help).
      IF mbfField._Help-SA NE ? THEN
        mchAtts = mchAtts + ',helpsa,' + Escapar(mbfField._Help-SA).
      IF mbfField._Initial-SA NE ? THEN
        mchAtts = mchAtts + ',initialsa,' + Escapar(mbfField._Initial-SA).
      IF mbfField._Valexp NE ? THEN
        mchAtts = mchAtts + ',valexp,' + Escapar(mbfField._Valexp).
      IF mbfField._ValMsg NE ? THEN
        mchAtts = mchAtts + ',valmsg,' + Escapar(mbfField._ValMsg).
      IF mbfField._ValMsg-SA NE ? THEN
        mchAtts = mchAtts + ',valmsgsa,' + Escapar(mbfField._ValMsg-SA).
      IF mbfField._Label NE ? THEN
        mchAtts = mchAtts + ',label,' + Escapar(mbfField._Label).
      IF mbfField._Label-SA NE ? THEN
        mchAtts = mchAtts + ',labelsa,' + Escapar(mbfField._Label-SA).
      IF mbfField._Format-SA NE ? THEN
        mchAtts = mchAtts + ',formatsa,' + Escapar(mbfField._Format-SA).
      IF mbfField._View-As NE ? THEN
        mchAtts = mchAtts + ',viewas,' + Escapar(mbfField._View-As). 
      IF mbfField._Decimals NE ? THEN
        mchAtts = mchAtts + ',decimals,' + STRING(mbfField._Decimals).
      IF mbfField._Extent NE ? THEN
        mchAtts = mchAtts  + ',extent,' + string(mbfField._Extent).
      IF mbfField._Width NE ? THEN
        mchAtts = mchAtts  + ',width,' +  string(mbfField._Width).
      IF mbfField._Field-Rpos NE ? THEN
        mchAtts = mchAtts  + ',position,' + string(mbfField._Field-Rpos).
      IF mbfField._Charset NE ? THEN
        mchAtts = mchAtts  + ',charset,' + mbfField._Charset.
      IF mbfField._Collation NE ? THEN
        mchAtts = mchAtts  + ',collation,' + mbfField._Collation.
      mchAtts = mchAtts + ',canread,' + mbfField._Can-Read + ',canwrite,' + mbfField._Can-Write +
                ',format,' + Escapar(mbfField._Format).
      OutputTag('field',mchAtts,?).
    end.                                                
  END METHOD.

  METHOD PROTECTED VOID AddTables():
    define buffer mbfFile for DictSource._File.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.

    /* ignorar tablas de sistema y estadsticas */
    for each  mbfFile NO-LOCK where mbfFile._File-num lt 32768 AND mbfFile._File-num GT 0 by mbfFile._File-num ON ERROR UNDO, THROW:
      DoLog('tabla ' + mbfFile._File-Name,'DEBUG').
      mchAtts = 'name,' + Escapar(mbfFile._File-Name) + 
        ',cancreate,' + mbfFile._Can-Create + ',candelete,' + mbfFile._Can-Delete +
        ',candump,' + mbfFile._Can-Dump + ',canload,' + mbfFile._Can-Load +
        ',canread,' + mbfFile._Can-Read + ',canwrite,' + mbfFile._Can-Write +
        ',creator,' + mbfFile._Creator + ',dblang,' + string(mbfFile._DB-Lang) +
        ',defpk,' + string(mbfFile._dft-Pk) + ',owner,' + mbfFile._Owner.
      IF mbfFile._Frozen THEN
        mchAtts = mchAtts + ',frozen,true'.
      IF mbfFile._Hidden THEN
        mchAtts = mchAtts + ',hidden,true'.
      IF mbfFile._Desc NE ? THEN
        mchAtts = mchAtts + ',desc,' + Escapar(mbfFile._Desc).
      IF mbfFile._Valexp NE ? THEN
        mchAtts = mchAtts + ',valexp,' + Escapar(mbfFile._Valexp).
      IF mbfFile._Valmsg NE ? THEN
        mchAtts = mchAtts + ',valmsg,' + Escapar(mbfFile._Valmsg).
      IF mbfFile._Valmsg-SA NE ? THEN
        mchAtts = mchAtts + ',valmsgsa,' + Escapar(mbfFile._Valmsg-SA).
      IF mbfFile._Dump-Name NE ? THEN
        mchAtts = mchAtts + ',dumpname,' + Escapar(mbfFile._Dump-Name).
      IF mbfFile._File-Label NE ? THEN
        mchAtts = mchAtts + ',label,' + Escapar(mbfFile._File-Label).
      IF mbfFile._File-Label-SA NE ? THEN
        mchAtts = mchAtts + ',salabel,' + Escapar(mbfFile._File-Label-SA).
      StartTag('table',mchAtts).
      AddStorageArea(mbfFile._File-num).
      DoLog('procesando campos..','DEBUG').
      addFields(recid(mbfFile)).
      DoLog('procesando ndices..','DEBUG').
      addIndexes(RECID(mbfFile),mbfFile._Prime-Index).
      EndTag().
    END.
  END METHOD.

  METHOD PROTECTED VOID AddSequences():
    define buffer mbfSequence for DictSource._Sequence.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.
    
    for each mbfSequence NO-LOCK ON ERROR UNDO, THROW:
      mchAtts = 'name,' + mbfSequence._Seq-Name.
      IF mbfSequence._Seq-Init NE ? THEN
        mchAtts = mchAtts + ',initial,' + STRING(mbfSequence._Seq-Init).
      IF mbfSequence._Seq-Num NE ? THEN
        mchAtts = mchAtts + ',number,' + string(mbfSequence._Seq-Num).
      IF mbfSequence._Seq-Incr NE ? THEN
        mchAtts = mchAtts + ',increment,' + STRING(mbfSequence._Seq-Incr).
      IF mbfSequence._Seq-Min NE ? THEN
        mchAtts = mchAtts + ',min,' + string(mbfSequence._Seq-Min).
      IF mbfSequence._Seq-Max NE ? THEN
        mchAtts = mchAtts + ',max,' + string(mbfSequence._Seq-Max).
      IF mbfSequence._Seq-Owner NE ? THEN
        mchAtts = mchAtts + ',owner,' + mbfSequence._Seq-Owner.
      IF mbfSequence._Cycle-Ok THEN
        mchAtts = mchAtts + ',cycle,true'.
      DoLog('secuencia ' + mbfSequence._Seq-Name,'DEBUG').
      OutputTag('sequence',mchAtts,?).
    end.
  END METHOD.

  METHOD PUBLIC OVERRIDE VOID writeXML(INPUT ipchDestino AS CHARACTER):
    define buffer mbfDB for DictSource._DB.
  
    StartXML('ddict','generador,NomadeSoft XRef',ipchDestino).
  
    IF cchDBName EQ ? THEN
      cchDBName = pdbname("DictSource").
  
    /* recid de la base de datos */
    find first mbfDB no-lock.
    criDB = recid(mbfDB).
  
    DoLog('Iniciando proceamiento para ' + cchDBName,'MESSAGE').
    StartTag('database','name,' + cchDBName).
    DoLog('Agregando tablas:','DEBUG').
    AddTables().
    DoLog('Agregando secuencias:','DEBUG').
    AddSequences().
    EndTag().
    DoLog('Definicin de ' + cchDBName + ' finalizada','MESSAGE').
    DoLog('Finalizando definicin','MESSAGE').
    FinishXML().
    CATCH e AS Progress.Lang.Error:
      DoLog(e:getMessage(1),'ERROR').
      IF e:CallStack NE ? THEN
        DoLog(e:CallStack,'DEBUG').
      DELETE OBJECT e.
    END CATCH.
  END METHOD.
END CLASS.
